from __future__ import annotations

TEXT_MAP_PATH = ".enka_py/assets/text_map_{lang}.json"
CHARACTER_DATA_PATH = ".enka_py/assets/characters.json"
NAMECARD_DATA_PATH = ".enka_py/assets/namecards.json"
CONSTS_DATA_PATH = ".enka_py/assets/consts.json"
TALENTS_DATA_PATH = ".enka_py/assets/talents.json"
PFPS_DATA_PATH = ".enka_py/assets/pfps.json"

SOURCE_TO_PATH = {
    "https://raw.githubusercontent.com/seriaati/enka-py-assets/main/data/text_map_{lang}.json": TEXT_MAP_PATH,
    "https://raw.githubusercontent.com/seriaati/enka-py-assets/main/data/characters.json": CHARACTER_DATA_PATH,
    "https://raw.githubusercontent.com/EnkaNetwork/API-docs/master/store/namecards.json": NAMECARD_DATA_PATH,
    "https://raw.githubusercontent.com/seriaati/enka-py-assets/main/data/consts.json": CONSTS_DATA_PATH,
    "https://raw.githubusercontent.com/seriaati/enka-py-assets/main/data/talents.json": TALENTS_DATA_PATH,
    "https://raw.githubusercontent.com/EnkaNetwork/API-docs/master/store/pfps.json": PFPS_DATA_PATH,
}
